package org.jboss.forge.addon.git;

import java.io.IOException;
import java.util.List;

import org.eclipse.jgit.api.CherryPickResult;
import org.eclipse.jgit.api.CreateBranchCommand.SetupUpstreamMode;
import org.eclipse.jgit.api.Git;
import org.eclipse.jgit.api.PullResult;
import org.eclipse.jgit.api.errors.GitAPIException;
import org.eclipse.jgit.api.errors.InvalidRefNameException;
import org.eclipse.jgit.api.errors.RefAlreadyExistsException;
import org.eclipse.jgit.api.errors.RefNotFoundException;
import org.eclipse.jgit.lib.Ref;
import org.eclipse.jgit.revwalk.RevCommit;
import org.eclipse.jgit.transport.FetchResult;
import org.jboss.forge.addon.git.exceptions.CantMergeCommitException;
import org.jboss.forge.addon.resource.DirectoryResource;
import org.jboss.forge.furnace.services.Exported;

@Exported
public interface GitUtils
{
   public Git clone(final DirectoryResource dir, final String repoUri) throws GitAPIException;

   public Git git(final DirectoryResource dir) throws IOException;

   public Ref checkout(final Git git, final String remote, final boolean createBranch,
            final SetupUpstreamMode mode, final boolean force)
            throws GitAPIException;

   public Ref checkout(final Git git, final Ref localRef, final boolean createBranch,
            final SetupUpstreamMode mode, final boolean force)
            throws GitAPIException;

   public FetchResult fetch(final Git git, final String remote, final String refSpec, final int timeout,
            final boolean fsck, final boolean dryRun,
            final boolean thin,
            final boolean prune) throws GitAPIException;

   /**
    * Initialize a new git repository.
    *
    * @param dir The directory in which to create a new .git/ folder and repository.
    */
   public Git init(final DirectoryResource dir) throws IOException;

   public PullResult pull(final Git git, final int timeout) throws GitAPIException;

   public List<Ref> getRemoteBranches(final Git repo) throws GitAPIException;

   public List<Ref> getLocalBranches(final Git repo) throws GitAPIException;

   public String getCurrentBranchName(final Git repo) throws IOException;

   public Ref switchBranch(final Git repo, final String branchName);

   public List<String> getLogForCurrentBranch(final Git repo) throws GitAPIException;

   public Iterable<RevCommit> getLogForBranch(final Git repo, String branchName) throws GitAPIException,
            IOException;

   public void add(final Git repo, String filepattern) throws GitAPIException;

   public void addAll(final Git repo) throws GitAPIException;

   public void commit(final Git repo, String message) throws GitAPIException;

   public void commitAll(final Git repo, String message) throws GitAPIException;

   public void stashCreate(final Git repo) throws GitAPIException;

   public void stashApply(final Git repo, String... stashRef) throws GitAPIException;

   public void stashDrop(final Git repo) throws GitAPIException;

   public void cherryPick(final Git repo, Ref commit) throws GitAPIException;

   public CherryPickResult cherryPickNoMerge(final Git git, Ref src) throws GitAPIException,
            CantMergeCommitException;

   public void resetHard(final Git repo, String newBase) throws GitAPIException;

   public Ref createBranch(Git git, String branchName) throws RefAlreadyExistsException, RefNotFoundException,
            InvalidRefNameException, GitAPIException;

   public void close(final Git repo);

}
