package org.jboss.forge.addon.git.ui;

import javax.inject.Inject;

import org.eclipse.jgit.api.CreateBranchCommand.SetupUpstreamMode;
import org.eclipse.jgit.api.Git;
import org.jboss.forge.addon.facets.constraints.FacetConstraint;
import org.jboss.forge.addon.git.GitUtils;
import org.jboss.forge.addon.git.facet.GitFacet;
import org.jboss.forge.addon.projects.Project;
import org.jboss.forge.addon.ui.UICommand;
import org.jboss.forge.addon.ui.context.UIBuilder;
import org.jboss.forge.addon.ui.context.UIContext;
import org.jboss.forge.addon.ui.input.UIInput;
import org.jboss.forge.addon.ui.input.UISelectOne;
import org.jboss.forge.addon.ui.metadata.WithAttributes;
import org.jboss.forge.addon.ui.result.Result;
import org.jboss.forge.addon.ui.result.Results;
import org.jboss.forge.addon.ui.util.Metadata;

@FacetConstraint(GitFacet.class)
public class GitCheckoutCommand extends AbstractGitCommand implements UICommand
{
   
   @Inject
   @WithAttributes(label = "Branch name", defaultValue = "master")
   private UIInput<String> branchInput;
   
   @Inject
   @WithAttributes(label = "Create branch")
   private UIInput<Boolean> createBranchInput;
   
   @Inject
   @WithAttributes(label = "Track", description = "Remote tracking mode")
   private UISelectOne<SetupUpstreamMode> modeInput;
   
   @Inject
   @WithAttributes(label = "Force")
   private UIInput<Boolean> forceInput;
   
   @Inject
   private GitUtils gitUtils;
   
   @Override
   public Metadata getMetadata()
   {
      return super.getMetadata().name("GIT: Checkout branch");
   }

   @Override
   public void initializeUI(UIBuilder builder) throws Exception
   {
      builder.add(branchInput).add(createBranchInput).add(modeInput).add(forceInput);
   }

   @Override
   public Result execute(UIContext context) throws Exception
   {
      Project project = getSelectedProject(context);
      Result result = null;
      if (project != null)
      {
         Git git = gitUtils.git(project.getProjectRoot());
         gitUtils.checkout(git, branchInput.getValue(), createBranchInput.getValue(), 
                  modeInput.getValue(), forceInput.getValue());
         gitUtils.close(git);
         result = Results.success();         
      } else {
         result = Results.fail("This command should be executed in the cotext of a project");
      }
      
      return result;
   }

}
