/*
 * Copyright 2012 Red Hat, Inc. and/or its affiliates.
 *
 * Licensed under the Eclipse Public License version 1.0, available at
 * http://www.eclipse.org/legal/epl-v10.html
 */

package org.jboss.forge.addon.git.ui;

import javax.inject.Inject;

import org.eclipse.jgit.api.Git;
import org.jboss.forge.addon.facets.constraints.FacetConstraint;
import org.jboss.forge.addon.git.GitUtils;
import org.jboss.forge.addon.git.facet.GitFacet;
import org.jboss.forge.addon.resource.DirectoryResource;
import org.jboss.forge.addon.resource.FileResource;
import org.jboss.forge.addon.ui.context.UIBuilder;
import org.jboss.forge.addon.ui.context.UIContext;
import org.jboss.forge.addon.ui.context.UIValidationContext;
import org.jboss.forge.addon.ui.input.UIInput;
import org.jboss.forge.addon.ui.metadata.WithAttributes;
import org.jboss.forge.addon.ui.result.Result;
import org.jboss.forge.addon.ui.result.Results;
import org.jboss.forge.addon.ui.util.Metadata;

/**
 * @author <a href="mailto:lincolnbaxter@gmail.com">Lincoln Baxter, III</a>
 * @author <a href="mailto:jevgeni.zelenkov@gmail.com">Jevgeni Zelenkov</a>
 *
 */
@FacetConstraint(GitFacet.class)
public class GitCloneCommand extends AbstractGitCommand
{

   @Inject
   @WithAttributes(label = "URI", description = "Git repository URI")
   private UIInput<String> uri;
   
   @Inject
   @WithAttributes(label = "Target directory")
   private UIInput<FileResource<?>> folderInput;
   
   @Inject
   private GitUtils gitUtils;
   
   @Override
   public Metadata getMetadata()
   {
      return super.getMetadata().name("GIT: Clone repository");
   }
   
   @Override
   public void initializeUI(UIBuilder builder) throws Exception
   {
      builder.add(uri).add(folderInput);
   }

   @Override
   public Result execute(UIContext context) throws Exception
   {
      FileResource<?> folder = folderInput.getValue();
      if (!folder.exists())
      {
         folder.mkdirs();
         folder = folder.reify(DirectoryResource.class);
      }
      Git clone = gitUtils.clone((DirectoryResource) folder, uri.getValue());
      gitUtils.close(clone);
      context.setSelection((FileResource<?>) folder);
      return Results.success();
   }

   @Override
   public void validate(UIValidationContext validator)
   {      
      FileResource<?> folder = folderInput.getValue();
      if (folder.exists() && !folder.isDirectory())
      {
         validator.addValidationError(folderInput, "The specified target directory must not be an existing file");
      }
   } 
   
   
}
