package org.jboss.forge.addon.git.ui;

import javax.inject.Inject;

import org.jboss.forge.addon.git.gitignore.resources.GitIgnoreResource;
import org.jboss.forge.addon.ui.context.UIBuilder;
import org.jboss.forge.addon.ui.context.UIContext;
import org.jboss.forge.addon.ui.input.UIInput;
import org.jboss.forge.addon.ui.metadata.WithAttributes;
import org.jboss.forge.addon.ui.result.Result;
import org.jboss.forge.addon.ui.result.Results;
import org.jboss.forge.addon.ui.util.Metadata;

public class GitIgnoreAddPatternCommand extends AbstractGitCommand
{

   @Inject
   @WithAttributes(label = "Pattern", required = true)
   private UIInput<String> patternInput;

   
   @Override
   public Metadata getMetadata()
   {
      return super.getMetadata().name("GITIGNORE: add pattern").description("Add pattern to .gitignore");
   }
   
   @Override
   public boolean isEnabled(UIContext context)
   {
      return super.isEnabled(context) && isGitIgnoreSelected(context);
   }

   @Override
   public void initializeUI(UIBuilder builder) throws Exception
   {
      builder.add(patternInput);
   }

   @Override
   public Result execute(UIContext context) throws Exception
   {
      GitIgnoreResource gitIgnore = (GitIgnoreResource) context.getInitialSelection().get();
      gitIgnore.addPattern(patternInput.getValue());

      return Results.success("Pattern added to the .gitignore in the current directory");
   }

}
