package org.jboss.forge.addon.git.ui;

import javax.inject.Inject;

import org.jboss.forge.addon.git.gitignore.resources.GitIgnoreResource;
import org.jboss.forge.addon.ui.context.UIBuilder;
import org.jboss.forge.addon.ui.context.UIContext;
import org.jboss.forge.addon.ui.input.InputComponent;
import org.jboss.forge.addon.ui.input.UICompleter;
import org.jboss.forge.addon.ui.input.UIInput;
import org.jboss.forge.addon.ui.metadata.WithAttributes;
import org.jboss.forge.addon.ui.result.Result;
import org.jboss.forge.addon.ui.result.Results;
import org.jboss.forge.addon.ui.util.Metadata;

public class GitIgnoreRemovePatternCommand extends AbstractGitCommand
{
   
   @Inject
   @WithAttributes(label = "Pattern", required = true)
   private UIInput<String> patternInput;
   
   @Override
   public Metadata getMetadata()
   {
      return super.getMetadata().name("GIT: remove pattern")
               .description("Remove pattern from .gitignore");
   }
   
   @Override
   public boolean isEnabled(UIContext context)
   {
      return super.isEnabled(context) && isGitIgnoreSelected(context);
   }

   @Override
   public void initializeUI(UIBuilder builder) throws Exception
   {
      patternInput.setCompleter(new GitIgnorePatternCompleter(gitIgnoreResource(builder.getUIContext())));
      builder.add(patternInput);
   }

   @Override
   public Result execute(UIContext context) throws Exception
   {
      GitIgnoreResource gitIgnore = (GitIgnoreResource) context.getInitialSelection().get();
      gitIgnore.removePattern(patternInput.getValue());

      return Results.success("Pattern removed from the .gitignore in the current directory");
   }

   private static class GitIgnorePatternCompleter implements UICompleter<String> 
   {
      private GitIgnoreResource resource;
      
      public GitIgnorePatternCompleter(GitIgnoreResource resource)
      {
         this.resource = resource;
      }

      @Override
      public Iterable<String> getCompletionProposals(UIContext context, InputComponent<?, String> input, String value)
      {
         return resource.getPatterns();
      }
      
   }
}
