/*
 * Copyright 2012 Red Hat, Inc. and/or its affiliates.
 *
 * Licensed under the Eclipse Public License version 1.0, available at
 * http://www.eclipse.org/legal/epl-v10.html
 */

package org.jboss.forge.addon.git.ui;

import javax.inject.Inject;

import org.jboss.forge.addon.git.gitignore.resources.GitIgnoreResource;
import org.jboss.forge.addon.resource.Resource;
import org.jboss.forge.addon.resource.ResourceFactory;
import org.jboss.forge.addon.ui.UICommand;
import org.jboss.forge.addon.ui.context.UIBuilder;
import org.jboss.forge.addon.ui.context.UIContext;
import org.jboss.forge.addon.ui.input.UIInput;
import org.jboss.forge.addon.ui.metadata.WithAttributes;
import org.jboss.forge.addon.ui.result.Result;
import org.jboss.forge.addon.ui.result.Results;
import org.jboss.forge.addon.ui.util.Metadata;

/**
 * @author Dan Allen
 */
public class LsGitIgnoreCommand extends AbstractGitCommand implements UICommand
{
   @Inject
   @WithAttributes(label = "Path")
   private UIInput<Resource<?>> resourceInput;
   
   @Inject
   private ResourceFactory resourceFactory;
   
   @Override
   public Metadata getMetadata()
   {
      return super.getMetadata().name("GIT: List .gitignore")
               .description("Prints the contents of a .gitignore file");
   }
   
   @Override
   public boolean isEnabled(UIContext context)
   {
      return super.isEnabled(context);
   }

   @Override
   public void initializeUI(UIBuilder builder) throws Exception
   {
      resourceInput.setDefaultValue(resourceFactory.create("."));
      builder.add(resourceInput);      
   }

   @Override
   public Result execute(UIContext context) throws Exception
   {
      StringBuilder sb = new StringBuilder();
      for (String pattern : ((GitIgnoreResource) resourceInput.getValue()).getPatterns())
      {
         sb.append(pattern).append("\n");
      }
      return Results.success(sb.toString());
   }
}
